/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.filesystem.cli.core;

import com.ibm.team.filesystem.cli.core.subcommands.IScmClientConfiguration;

/**
 * The command-line environment. Convenience class allowing access to the 
 * {@link IClientConfiguration}. 
 */
public abstract class CommandLineCore {
	private static ThreadLocal<IScmClientConfiguration> config = new ThreadLocal<IScmClientConfiguration>();
	
	/** Get the configuration we're running with. Note that this may return 
	 * <code>null</code> if we're running outside of the normal subcommand 
	 * lifecycle (ie, in JUnits).
	 */
	@SuppressWarnings("unchecked")
	public static <T extends IScmClientConfiguration> T getConfig() {
		return (T)config.get();
	}

	/**
	 * Assign the config for the current thread. 
	 */
	public static <T extends IScmClientConfiguration> void setConfig(T newValue) {
		if (newValue != null && config.get() != null) {
			throw new IllegalStateException("May not reassign config"); //$NON-NLS-1$
		}
		
		config.set(newValue);
	}
}
