/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2015. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.filesystem.cli.core.internal;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.logging.Log;

import com.ibm.team.repository.common.LogFactory;
import com.ibm.team.rtc.cli.infrastructure.internal.parser.CommandLineArgument;
import com.ibm.team.rtc.cli.infrastructure.internal.parser.ICommandLineArgument;
import com.ibm.team.rtc.cli.infrastructure.internal.parser.IOptionKey;
import com.ibm.team.filesystem.cli.core.Constants;
import com.ibm.team.filesystem.cli.core.subcommands.CommonOptions;
import com.ibm.team.filesystem.cli.core.subcommands.IScmClientConfiguration;
import com.ibm.team.filesystem.cli.core.subcommands.IScmCommandLineArgument;
import com.ibm.team.filesystem.cli.core.subcommands.IUuidAliasRegistry.IUuidAlias;
import com.ibm.team.filesystem.cli.core.util.RepoUtil;

public class ScmCommandLineArgument extends CommandLineArgument implements IScmCommandLineArgument {
	private static final Log log = LogFactory.getLog(ScmCommandLineArgument.class.getName());
	
	String repo;
	boolean repoExplicit;
	IUuidAlias uuidAlias;
	boolean isAliasValidated;
	boolean checkAlias;
	
	public ScmCommandLineArgument(IOptionKey optionKey, String arg, String prefix, String suffix, IScmClientConfiguration config, boolean checkAlias) {
		
		this(optionKey, arg, prefix, suffix, config);
		
		this.checkAlias = checkAlias;	
	}
	
	
	public ScmCommandLineArgument(IOptionKey optionKey, String arg, String prefix, String suffix, IScmClientConfiguration config) {
        super(optionKey, arg, prefix, suffix);

        if (suffix == null || suffix.length() == 0) {
            repoExplicit = false;
            repo = null;
            
            if (config.getSubcommandCommandLine().hasOption(CommonOptions.OPT_URI)) {
                repo = config.getSubcommandCommandLine().getOption(CommonOptions.OPT_URI);
            }
        } else {
            repoExplicit = true;
            repo = suffix;
        }
        
        this.uuidAlias = null;
        this.isAliasValidated = false;
	}
	
	/**
	 * @param arg
	 * @param config
	 */
	public static ScmCommandLineArgument create(ICommandLineArgument arg, IScmClientConfiguration config) {
	    return create(arg, config, true);
	}
	
	public static ScmCommandLineArgument create(ICommandLineArgument arg, IScmClientConfiguration config, boolean checkAlias) {
	    if (arg == null) {
	        return null;
	    }

	    return new ScmCommandLineArgument(arg.getKey(), arg.getValue(), arg.getValuePrefix(), arg.getValueSuffix(), config, checkAlias);
	}
    /**
     * @param arg
     * @param config
     */
    public static List<IScmCommandLineArgument> createList(List<ICommandLineArgument> args,
            IScmClientConfiguration config) {             
        return createList(args, config, true);
    }
    
    public static List<IScmCommandLineArgument> createList(List<ICommandLineArgument> args,
            IScmClientConfiguration config, boolean checkAlias) {
        List<IScmCommandLineArgument> scmArgs = new ArrayList<IScmCommandLineArgument>(args.size());
        for (ICommandLineArgument arg : args) {
            IScmCommandLineArgument scmArg = ScmCommandLineArgument.create(arg, config, checkAlias);
            if (scmArg != null) {
                scmArgs.add(scmArg);
            }
        }
        
        return scmArgs;
    }
	
	public String getItemSelector() {
		return getValuePrefix();
	}
	
	public String getRepositorySelector() {
		validateAlias();
		return repo;
	}
	
	public boolean isRepoExplicit() {
		validateAlias();
		return repoExplicit;
	}
	
	public String getStringValue() {
		if (repoExplicit) {
			return new String(getItemSelector() + Constants.AT_STR + repo);
		}
		
		return getItemSelector();
	}
	
	public IUuidAlias getAlias() {
		validateAlias();
		return uuidAlias;
	}

	private void validateAlias() {
		if (checkAlias && !isAliasValidated) {
			uuidAlias = RepoUtil.lookupUuidAndAlias(getItemSelector(), repo);
			if (uuidAlias != null && uuidAlias.getRepositoryId() != null) {
				if (repoExplicit) {
					log.info("Ignoring repo information for alias " + getItemSelector()); //$NON-NLS-1$
				}
				
				repo = uuidAlias.getRepositoryUrl(); // use the repo that we have stored with the alias and ignore other repo info, if any
				repoExplicit = true; // treat it as explicitly specified by the user as we may have to login to this repo later
			}
			
			isAliasValidated = true;
		}
	}
	
	public void setRepositorySelector(String repo) {
		this.repo = repo;
		this.repoExplicit = true;
	}
}
