/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.filesystem.cli.core.internal.aliases;

import com.ibm.team.filesystem.cli.core.subcommands.IUuidAliasRegistry.IUuidAlias;
import com.ibm.team.filesystem.cli.core.util.RepoUtil.ItemType;
import com.ibm.team.repository.common.UUID;

public class UuidAlias implements IUuidAlias {
	private final String monicker;
	final UUID uuid;
	private final String repositoryUrl;
	private final UUID repositoryId;
	long lastRun;
    ItemType itemType;
	boolean persist = true;

	UuidAlias(String m, UUID u, String r, UUID rId, long c, ItemType t) {
		this.monicker = m;
		this.uuid = u;
		this.repositoryUrl = r;
		this.repositoryId = rId;
		this.lastRun = c; 
		this.itemType = t;
	}
	
	public UuidAlias(UUID u, String r, UUID rId) {
		this.monicker = null;
		this.uuid = u;
		this.repositoryUrl = r;
		this.repositoryId = rId;
		this.lastRun = 0; 
		this.itemType = ItemType.UNKNOWN;
	}

	public String getMonicker() {
		return monicker;
	}

	public UUID getUuid() {
		return uuid;
	}

	public String getRepositoryUrl() {
		return repositoryUrl;
	}

	public UUID getRepositoryId() {
		return repositoryId;
	}
	
	public ItemType getItemType() {
	    return itemType;
	}

	@Override
	public boolean equals(Object o) {
		if (o == this) {
			return true;
		}

		if (o instanceof IUuidAlias) {
			IUuidAlias other = (IUuidAlias) o;

			return monicker.equals(other.getMonicker())
					&& uuid.equals(other.getUuid());
		}

		return false;
	}

	@Override
	public int hashCode() {
		return monicker.hashCode() ^ uuid.hashCode();
	}

	public long getLastRun() {
		return lastRun;
	}
	
	public UuidAlias createCopyForSave() {
		return new UuidAlias(monicker, uuid, repositoryUrl, repositoryId, lastRun, itemType);
	}
}