/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/

package com.ibm.team.filesystem.cli.core.internal.compare;


/**
 * The external compare tool "Beyond Compare". (http://www.scootersoftware.com).
 */
public class CompareToolBeyondCompare implements IExternalCompareTool {

	public static final String EXTERNAL_TOOL_NAME = "Beyond Compare"; //$NON-NLS-1$
	private final String WINDOWS_X32_DEFAULT_2_WAY_COMPARE_EXECUTABLE_PATH = "C:\\Program Files\\Beyond Compare 3\\BComp.exe"; //$NON-NLS-1$
	private final String WINDOWS_X64_DEFAULT_2_WAY_COMPARE_EXECUTABLE_PATH = "C:\\Program Files (x86)\\Beyond Compare 3\\BComp.exe"; //$NON-NLS-1$
	private final String WINDOWS_X32_DEFAULT_3_WAY_MERGE_EXECUTABLE_PATH = "C:\\Program Files\\Beyond Compare 3\\BComp.exe"; //$NON-NLS-1$
	private final String WINDOWS_X64_DEFAULT_3_WAY_MERGE_EXECUTABLE_PATH = "C:\\Program Files (x86)\\Beyond Compare 3\\BComp.exe"; //$NON-NLS-1$
	private final String NON_WINDOWS_DEFAULT_2_WAY_COMPARE_EXECUTABLE_PATH = "bcompare"; //$NON-NLS-1$
    private final String NON_WINDOWS_DEFAULT_3_WAY_MERGE_EXECUTABLE_PATH = "bcompare"; //$NON-NLS-1$
	
	// Arguments
	private final String ARGUMENT_LOCAL_FILE_TITLE = ExternalCompareToolsUtil.getCmdLineParameterPrefix() + "title1=\"" + LOCAL_FILE_TITLE + "\""; //$NON-NLS-1$ //$NON-NLS-2$
	private final String ARGUMENT_REMOTE_FILE_TITLE = ExternalCompareToolsUtil.getCmdLineParameterPrefix() + "title2=\"" + REMOTE_FILE_TITLE + "\""; //$NON-NLS-1$ //$NON-NLS-2$
	private final String ARGUMENT_ANCESTOR_FILE_TITLE = ExternalCompareToolsUtil.getCmdLineParameterPrefix() + "title3=\"" + ANCESTOR_FILE_TITLE + "\""; //$NON-NLS-1$ //$NON-NLS-2$
	private final String ARGUMENT_MERGE_FILE_TITLE = ExternalCompareToolsUtil.getCmdLineParameterPrefix() + "title4=\"" + MERGE_FILE_TITLE + "\""; //$NON-NLS-1$ //$NON-NLS-2$
	private final String ARGUMENT_LEFT_READ_ONLY = ExternalCompareToolsUtil.getCmdLineParameterPrefix() + "leftreadonly"; //$NON-NLS-1$
	private final String ARGUMENT_RIGHT_READ_ONLY = ExternalCompareToolsUtil.getCmdLineParameterPrefix() + "rightreadonly"; //$NON-NLS-1$
	
	private static CompareToolBeyondCompare instance = null;
    
    private CompareToolBeyondCompare() {      
    }
    
    public static synchronized CompareToolBeyondCompare getInstance() {
        if (instance == null) {
            instance = new CompareToolBeyondCompare();
        }        
        return instance;
    }
	
	@Override
    public String getName() {
		return EXTERNAL_TOOL_NAME;
	}

	
	// 2 Way Diff Methods:
	
	@Override
    public boolean doesToolSupport2WayCompare() {
		return true;
	}


    @Override
    public String getDefault2WayCompareExecutablePath() {
        String path = ""; //$NON-NLS-1$
        if (ExternalCompareToolsUtil.isMSWindows()) {
            if (ExternalCompareToolsUtil.isMSWindows64Bit()) {
                path = WINDOWS_X64_DEFAULT_2_WAY_COMPARE_EXECUTABLE_PATH;
            } else {
                path = WINDOWS_X32_DEFAULT_2_WAY_COMPARE_EXECUTABLE_PATH;
            }
        } else {
            path = NON_WINDOWS_DEFAULT_2_WAY_COMPARE_EXECUTABLE_PATH;
        }
        return path;
    }


	@Override
    public String getDefault2WayLocalCompareArguments() {
		String twoWayLocalCompareArgs;
		if (ExternalCompareToolsUtil.isMSWindows()) {
		    twoWayLocalCompareArgs = ExternalCompareToolsUtil.addQuotes(LOCAL_FILE_PATH) + " " + //$NON-NLS-1$
		                             ExternalCompareToolsUtil.addQuotes(REMOTE_FILE_PATH) + " " + //$NON-NLS-1$
                                     ARGUMENT_LOCAL_FILE_TITLE + " " + //$NON-NLS-1$
                                     ARGUMENT_REMOTE_FILE_TITLE + " " + //$NON-NLS-1$
                                     ARGUMENT_RIGHT_READ_ONLY;
		} else {
            twoWayLocalCompareArgs = LOCAL_FILE_PATH + " " + //$NON-NLS-1$
                                     REMOTE_FILE_PATH + " " + //$NON-NLS-1$
                                     ARGUMENT_LOCAL_FILE_TITLE + " " + //$NON-NLS-1$
                                     ARGUMENT_REMOTE_FILE_TITLE + " " + //$NON-NLS-1$
                                     ARGUMENT_RIGHT_READ_ONLY;
		}
		return twoWayLocalCompareArgs;
	}

	
	@Override
    public String getDefault2WayRemoteCompareArguments() {
        String twoWayRemoteCompareArgs;
        if (ExternalCompareToolsUtil.isMSWindows()) {
            twoWayRemoteCompareArgs = ExternalCompareToolsUtil.addQuotes(LOCAL_FILE_PATH) + " " + //$NON-NLS-1$
                                      ExternalCompareToolsUtil.addQuotes(REMOTE_FILE_PATH) + " " + //$NON-NLS-1$
                                      ARGUMENT_LOCAL_FILE_TITLE + " " + //$NON-NLS-1$
                                      ARGUMENT_REMOTE_FILE_TITLE + " " + //$NON-NLS-1$
                                      ARGUMENT_LEFT_READ_ONLY + " " + //$NON-NLS-1$
                                      ARGUMENT_RIGHT_READ_ONLY;
        } else {
            twoWayRemoteCompareArgs = LOCAL_FILE_PATH + " " + //$NON-NLS-1$
                                      REMOTE_FILE_PATH + " " + //$NON-NLS-1$
                                      ARGUMENT_LOCAL_FILE_TITLE + " " + //$NON-NLS-1$
                                      ARGUMENT_REMOTE_FILE_TITLE + " " + //$NON-NLS-1$
                                      ARGUMENT_LEFT_READ_ONLY + " " + //$NON-NLS-1$
                                      ARGUMENT_RIGHT_READ_ONLY; 
        }
        return twoWayRemoteCompareArgs;
    }

	@Override
    public String get2WayCompareExecutablePathPrefDefaultValue() {
		return getDefault2WayCompareExecutablePath();
	}
	
    @Override
    public String get2WayLocalCompareArgumentsPrefDefaultValue() {
        return getDefault2WayLocalCompareArguments();
    }
    
    @Override
    public String get2WayRemoteCompareArgumentsPrefDefaultValue() {
        return getDefault2WayRemoteCompareArguments();
    }
	

	// 3 Way Merge Methods:
	
	@Override
    public boolean doesToolSupport3WayMerge() {
		return true;
	}


    @Override
    public String getDefault3WayMergeExecutablePath() {
        String path = ""; //$NON-NLS-1$
        if (ExternalCompareToolsUtil.isMSWindows()) {
            if (ExternalCompareToolsUtil.isMSWindows64Bit()) {
                path = WINDOWS_X64_DEFAULT_3_WAY_MERGE_EXECUTABLE_PATH;
            } else {
                path = WINDOWS_X32_DEFAULT_3_WAY_MERGE_EXECUTABLE_PATH;
            }
        } else {
            path = NON_WINDOWS_DEFAULT_3_WAY_MERGE_EXECUTABLE_PATH;
        }
        return path;
    }


	@Override
    public String getDefault3WayMergeArguments() {
		String mergeArguments;
		if (ExternalCompareToolsUtil.isMSWindows()) {
		    mergeArguments = ExternalCompareToolsUtil.addQuotes(LOCAL_FILE_PATH) + " " + //$NON-NLS-1$
		                     ExternalCompareToolsUtil.addQuotes(REMOTE_FILE_PATH) + " " + //$NON-NLS-1$
		                     ExternalCompareToolsUtil.addQuotes(ANCESTOR_FILE_PATH) + " " + //$NON-NLS-1$
		                     ExternalCompareToolsUtil.addQuotes(MERGE_FILE_PATH) + " " + //$NON-NLS-1$
                		     ARGUMENT_LOCAL_FILE_TITLE + " " + //$NON-NLS-1$
                			 ARGUMENT_REMOTE_FILE_TITLE + " " + //$NON-NLS-1$
                			 ARGUMENT_ANCESTOR_FILE_TITLE + " " + //$NON-NLS-1$
                			 ARGUMENT_MERGE_FILE_TITLE;
		} else {
		    mergeArguments = LOCAL_FILE_PATH + " " + //$NON-NLS-1$
                             REMOTE_FILE_PATH + " " + //$NON-NLS-1$
                             ANCESTOR_FILE_PATH + " " + //$NON-NLS-1$
                             MERGE_FILE_PATH + " " + //$NON-NLS-1$
                             ARGUMENT_LOCAL_FILE_TITLE + " " + //$NON-NLS-1$
                             ARGUMENT_REMOTE_FILE_TITLE + " " + //$NON-NLS-1$
                             ARGUMENT_ANCESTOR_FILE_TITLE + " " + //$NON-NLS-1$
                             ARGUMENT_MERGE_FILE_TITLE;
		}
		return mergeArguments;
	}
	
	@Override
    public String get3WayMergeExecutablePathPrefDefaultValue() {
		return getDefault3WayMergeExecutablePath();
	}

    @Override
    public String get3WayMergeArgumentsPrefDefaultValue() {
        return getDefault3WayMergeArguments();
    }
}
