/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.filesystem.cli.minimal.util;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.LinkedHashMap;
import java.util.regex.Matcher;
import java.util.regex.Pattern;


public class TestRunTimeComparator {

	private static class TestResult {
		final double duration;
		final boolean passed;
		
		TestResult(double t, boolean p) {
			duration = t;
			passed = p;
		}
	}
	
	private static class TestRun {
		final LinkedHashMap<String, TestResult> tests = new LinkedHashMap<String, TestResult>();
		final File file;
		
		public TestRun(File file) {
			this.file = file;
		}
	}
	
	public static void main(String[] args) throws Exception {
		if (args.length != 3) {
			System.err.println("Expected three arguments: test1.xml test2.xml outputfile.csv");
		}
		
		File test1 = new File(args[0]);
		File test2 = new File(args[1]);
		File output = new File(args[2]);
		
		TestRun t1 = parse(test1);
		TestRun t2 = parse(test2);
		
		writeComparison(t1, t2, output);
	}

	private static void writeComparison(TestRun t1, TestRun t2, File output) throws IOException {
		FileWriter writer = new FileWriter(output);
		
		try {
			writer.write("Test, ");
			writer.write(t1.file.getName());
			writer.write(", ");
			writer.write(t2.file.getName());
			writer.write(", \n");
			
			for (String test : t1.tests.keySet()) {
				double diff = t1.tests.get(test).duration - t2.tests.get(test).duration;
				String pf = (t1.tests.get(test).passed ? "p" : "f")
							+ (t2.tests.get(test).passed ? "p" : "f");
				
				writer.write(test + ", " + t1.tests.get(test).duration + ", " + t2.tests.get(test).duration + ", " + diff + ", " + pf + "\n");
			}
		} finally {
			writer.close();
		}
	}

	private static TestRun parse(File test) throws IOException {
		Pattern p = Pattern.compile("<testcase name=\"([^\"]+)\" classname=\"([^\"]+)\" time=\"([^\"]+)\"(/?)>");
		
		BufferedReader reader = new BufferedReader(new FileReader(test));
		
		try {
			TestRun testRun = new TestRun(test);
			
			String line;
			while (null != (line = reader.readLine())) {
				Matcher m = p.matcher(line.trim());
				
				if (m.matches()) {
					String name = m.group(2) + "." + m.group(1);
					TestResult result = new TestResult(Double.parseDouble(m.group(3)), m.group(4).length() == 1);
					testRun.tests.put(name, result);
				}
			}
			
			return testRun;
		} finally {
			reader.close();
		}
	}
	
}
