/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.filesystem.cli.tools.dump;

import java.io.IOException;
import java.io.LineNumberReader;

/**
 * Acts like a {@link LineNumberReader}, but emits an EOF when we get to two 
 * newlines. 
 */
public class SectioningLineNumberReader {
	private final LineNumberReader wrapped;
	
	private boolean foundSectionTerminator = false;

	SectioningLineNumberReader(LineNumberReader r) {
		this.wrapped = r;
	}
	
	public String readLine() throws IOException {
		String line = wrapped.readLine();
		if (line == null) {
			return null;
		}
		
		if (line.trim().length() == 0) {
			foundSectionTerminator = true;
			return null;
		}
		
		return line;
	}
	
	public int getLineNumber() {
		return wrapped.getLineNumber();
	}
	
	boolean foundSectionTerminator() {
		return foundSectionTerminator;
	}
}