/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.filesystem.cli.tools.dump.blockparsers;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import com.ibm.team.filesystem.cli.tools.dump.MetadataDumpVisitor;
import com.ibm.team.filesystem.cli.tools.dump.SectioningLineNumberReader;
import com.ibm.team.filesystem.client.internal.copyfileareas.metadata.validator.MetaDatasCollector;

/**
 * Base class for the various different blocks we expect to see in a metadata
 * dump. 
 */
public abstract class BlockParser {

	public BlockParser() {
		super();
	}

	/**
	 * @return The text of the section heading that acts as a preamble to the 
	 * block. 
	 */
	public abstract String getSectionHeading();

	/**
	 * Calls the appropriate visitor method.  
	 * @param c The visitor. 
	 * @param buf The original text of the metadata. 
	 * @param values Name/value pairs picked up during the parse.  
	 */
	protected abstract void visit(MetadataDumpVisitor c, StringBuffer buf, Map<String, String> values);

	/**
	 * Perform a parse of the metadata from <tt>reader</tt>. Implementors 
	 * should expect <tt>reader</tt> to return <code>null</code> when the end 
	 * of the metadata block is reached. 
	 * 
	 * @param reader The source of the input. 
	 * @param c The visitor that should receive calls when fields are read from
	 * 		the metadata stream. 
	 */
	public void parse(SectioningLineNumberReader reader, MetadataDumpVisitor c) throws IOException {
		Map<String, String> values = new HashMap<String, String>();
		StringBuffer buf = new StringBuffer();
		
		String line;
		while (null != (line = reader.readLine())) {
			if (MetaDatasCollector.TERMINATOR.equals(line)) {
				
				visit(c, buf, values);
				
				buf = new StringBuffer();
				values = new HashMap<String, String>();
			}
			else {
				String[] pair = line.split(":", 2);
				values.put(pair[0].trim(), pair[1].trim());
				buf.append(line);
				buf.append("\n");
			}
		}
	}
}