/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.filesystem.cli.tools.dump.blockparsers;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.ibm.team.filesystem.cli.tools.dump.MetadataDumpVisitor;
import com.ibm.team.filesystem.cli.tools.dump.SectioningLineNumberReader;
import com.ibm.team.filesystem.cli.tools.dump.blocks.InverseMetadataBlock;
import com.ibm.team.filesystem.cli.tools.dump.blocks.InverseMetadataBlock.RemoteChild;
import com.ibm.team.filesystem.client.internal.copyfileareas.metadata.validator.MetaDatasCollector;
import com.ibm.team.repository.common.UUID;

public class InverseMetadataDumpParser extends BlockParser {
	/**
	 * Note that the only state that's explicitly used is 
	 * {@link #REMOTE_CHILDREN} - that's because the other states are treated 
	 * as colon-separated name-value-pairs. 
	 */
	enum ParseState {BODY, ORIGINAL_PROPS, CHANGED_PROPS, REMOTE_CHILDREN};
	
	/* We override #parse so we can treat our properties and 
	 * children specially. 
	 */
	@SuppressWarnings("null")
	public void parse(SectioningLineNumberReader reader, MetadataDumpVisitor c) throws IOException {
		ParseState state = ParseState.BODY;
		
		Map<String, String> bodyValues = new HashMap<String, String>();
        StringBuffer buf = new StringBuffer();
        
        StringBuffer changedProps = null;
        StringBuffer originalProps = null;
        StringBuffer remoteChildrenBuf = null;
        
        Map<String, String> changedPropsMap = null;
        Map<String, String> originalPropsMap = null;
        List<RemoteChild> remoteChildren = null;
        
        
        /* These two current fields are pointers to one of the families 
         * of values above. They get adjusted to different maps/buffers 
         * depending on the current state.    
         */
        StringBuffer curBuf = buf;
        Map<String, String> curMap = bodyValues;
        
        String line;
        while (null != (line = reader.readLine())) {
            if (MetaDatasCollector.TERMINATOR.equals(line)) {
                InverseMetadataBlock block = new InverseMetadataBlock(
                		buf.toString(),
                		originalPropsMap,
                		changedPropsMap,
                		remoteChildren,
                		UUID.valueOf(bodyValues.get("item ID")),
                		UUID.valueOf(bodyValues.get("component ID")),
                		UUID.valueOf(bodyValues.get("connection ID"))
        		);
                
                c.consumeInverseMetadataBlock(block);
            }
            else if (MetaDatasCollector.SUBSECTION_ORIGINAL_PROPERTIES.equals(line)) {
                originalProps = new StringBuffer();
                curBuf = originalProps;
                
                originalPropsMap = new HashMap<String, String>();
                curMap = originalPropsMap;
                
                state = ParseState.ORIGINAL_PROPS;
            }
            else if (MetaDatasCollector.SUBSECTION_CHANGED_PROPERTIES.equals(line)) {
                changedProps = new StringBuffer();
                curBuf = changedProps;
                
                changedPropsMap = new HashMap<String, String>();
                curMap = changedPropsMap;
                
                state = ParseState.CHANGED_PROPS;
            }
            else if (MetaDatasCollector.SUBSECTION_REMOTE_CHILDREN.equals(line)) {
            	remoteChildren = new ArrayList<RemoteChild>();
                remoteChildrenBuf = new StringBuffer();
                curBuf = remoteChildrenBuf;
                state = ParseState.REMOTE_CHILDREN;
            }
            else {
                if (state == ParseState.REMOTE_CHILDREN) {
                	String[] rc = line.split("/", 2);
                	RemoteChild remoteChild = new RemoteChild(line, UUID.valueOf(rc[1].trim()), rc[0].trim());
                	remoteChildren.add(remoteChild);
                }
                else {
                	String[] pair = line.split(":", 2);
                	
                	curMap.put(pair[0].trim(), pair[1].trim());
                }
                
                curBuf.append(line);
                curBuf.append("\n");
            }
        }
    }

	@Override
	public String getSectionHeading() {
		return MetaDatasCollector.SECTION_INVERSE_INFO;
	}

	@Override
	protected void visit(MetadataDumpVisitor c, StringBuffer buf, Map<String, String> values) {
		throw new IllegalStateException("parse() should be redefined and shouldn't call this.");
	}
}
