/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.filesystem.cli.tools.dump.blocks;

import java.io.PrintWriter;
import java.util.Comparator;

import com.ibm.team.filesystem.cli.tools.dump.MetadataDumpParser.MetadataBlock;
import com.ibm.team.filesystem.client.internal.copyfileareas.metadata.validator.MetaDatasCollector;
import com.ibm.team.repository.common.UUID;

/**
 * See 
 * com.ibm.team.filesystem.client.internal.SharingDescriptorsMap
 */
public class ForwardSharingMetadataBlock extends MetadataBlock {
	public static final Comparator<ForwardSharingMetadataBlock> COMPARATOR = new Comparator<ForwardSharingMetadataBlock>() {
        @Override
        public int compare(ForwardSharingMetadataBlock object1, ForwardSharingMetadataBlock object2) {
            return object1.getPath().compareTo(object2.getPath());
        }
    };
	
	private final String path;                
	private final String componentName;      
	private final String connectionName;   
	
	private final UUID componentID;   
	private final UUID connectionID;  
	private final UUID connectionStateID; 
	private final UUID repositoryID;       
	private final UUID versionableID; 
	private final UUID versionableStateID;
	
	public ForwardSharingMetadataBlock(String self, String path,
			String componentName, String connectionName, 
			UUID componentID,
			UUID connectionID, UUID connectionStateID, UUID repositoryID,
			UUID versionableID, UUID versionableStateID) {
		super(self);
		this.path = path;
		this.componentName = componentName;
		this.connectionName = connectionName;
		this.componentID = componentID;
		this.connectionID = connectionID;
		this.connectionStateID = connectionStateID;
		this.repositoryID = repositoryID;
		this.versionableID = versionableID;
		this.versionableStateID = versionableStateID;
	}

	public String getSelf() {
		return self;
	}

	public String getPath() {
		return path;
	}

	public String getComponentName() {
		return componentName;
	}

	public String getConnectionName() {
		return connectionName;
	}

	public UUID getComponentID() {
		return componentID;
	}

	public UUID getConnectionID() {
		return connectionID;
	}

	public UUID getConnectionStateID() {
		return connectionStateID;
	}

	public UUID getRepositoryID() {
		return repositoryID;
	}

	public UUID getVersionableID() {
		return versionableID;
	}

	public UUID getVersionableStateID() {
		return versionableStateID;
	}

	@Override
	protected void dumpHeading(PrintWriter w) {
		w.println(MetaDatasCollector.SECTION_SHARING_INFO);
	}
}