/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.filesystem.cli.tools.dump.blocks;

import java.io.PrintWriter;
import java.util.Comparator;

import com.ibm.team.filesystem.cli.tools.dump.MetadataDumpParser.MetadataBlock;
import com.ibm.team.filesystem.client.internal.copyfileareas.metadata.validator.MetaDatasCollector;
import com.ibm.team.repository.common.UUID;

/**
 * Analogue for 
 * com.ibm.team.filesystem.client.internal.core.SharingMetadata2.InverseSharingDescriptorMap.InverseSharingDescriptorMap
 */
public class InverseSharingMetadataBlock extends MetadataBlock implements IScopedByComponentMetadataBlock {
	
	public static final Comparator<InverseSharingMetadataBlock> COMPARATOR = new Comparator<InverseSharingMetadataBlock>() {
		@Override
		public int compare(InverseSharingMetadataBlock object1, InverseSharingMetadataBlock object2) {
		    int r = IScopedByComponentMetadataBlock.COMPARATOR.compare(object1, object2);
		    if (r == 0) {
		    	r = object1.getItemId().compareTo(object2.getItemId());
		    }
		    
		    return r;
		}
    };
	
	private final UUID connectionId;
	private final UUID componentId;
	private final String connectionType;
	private final String connectionTypeNS;
	
	private final UUID itemId;

	public InverseSharingMetadataBlock(String self, UUID connectionId, UUID componentId, String connectionType, String connectionTypeNS, UUID itemId) {
		super(self);
		
		this.connectionId = connectionId;
		this.componentId = componentId;
		this.connectionType = connectionType;
		this.connectionTypeNS = connectionTypeNS;
		this.itemId = itemId;
	}

	@Override
	public UUID getConnectionId() {
		return connectionId;
	}

	@Override
	public UUID getComponentId() {
		return componentId;
	}

	public String getConnectionType() {
		return connectionType;
	}

	public String getConnectionTypeNS() {
		return connectionTypeNS;
	}

	public UUID getItemId() {
		return itemId;
	}

	@Override
	protected void dumpHeading(PrintWriter w) {
		w.println(MetaDatasCollector.SECTION_INVERSE_SHARING_INFO);
	}
	
	
}