/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.filesystem.cli.tools.verify;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

import com.ibm.team.filesystem.client.FileSystemException;
import com.ibm.team.filesystem.client.rest.IFilesystemRestClient;
import com.ibm.team.repository.client.ITeamRepository;
import com.ibm.team.rtc.cli.infrastructure.internal.core.IOptionSource;
import com.ibm.team.rtc.cli.infrastructure.internal.parser.ICommandLine;
import com.ibm.team.rtc.cli.infrastructure.internal.parser.IOptionKey;
import com.ibm.team.rtc.cli.infrastructure.internal.parser.OptionKey;
import com.ibm.team.rtc.cli.infrastructure.internal.parser.Options;
import com.ibm.team.rtc.cli.infrastructure.internal.parser.PositionalOptionDefinition;
import com.ibm.team.rtc.cli.infrastructure.internal.parser.exceptions.ConflictingOptionException;
import com.ibm.team.rtc.cli.infrastructure.internal.util.IndentingPrintStream;
import com.ibm.team.rtc.cli.infrastructure.internal.util.StringUtil;
import com.ibm.team.filesystem.cli.core.AbstractSubcommand;
import com.ibm.team.filesystem.cli.core.internal.PreferenceRegistry;
import com.ibm.team.filesystem.cli.core.util.RepoUtil;
import com.ibm.team.filesystem.cli.core.util.SubcommandUtil;
import com.ibm.team.scm.client.SCMPlatform;

public class ConfigValueCmd extends AbstractSubcommand implements IOptionSource {
    
	private static final IOptionKey OPT_KEY = new OptionKey("keys");

    public Options getOptions() throws ConflictingOptionException {
        Options options = new Options(false);
        options.setLongHelp("TOOLS: Outputs values of configurations.");

        SubcommandUtil.addRepoLocationToOptions(options);
        
        List<String> keys = new ArrayList<String>();
        keys.add(PreferenceRegistry.PREF_CONTENT_TRANSFER_THREADS);
        keys.add(PreferenceRegistry.PREF_REPO_CONNECTION_TIMEOUT);
        StringBuilder builder = new StringBuilder();
        for (String key : keys) {
        	if (builder.length() > 0) {
        		builder.append(", ");
        	}
        	builder.append(key);
        }
        options.addOption(new PositionalOptionDefinition(OPT_KEY, "keys", 0, Options.ARG_INFINITE),
        		"The configuration to query, or ignore this field to print all values. Keys: " + builder.toString());

        return options;
    }
    
	public void run() throws FileSystemException {
	    IFilesystemRestClient client = SubcommandUtil.setupDaemon(config);
		ITeamRepository repo = RepoUtil.login(config, client, config.getConnectionInfo());

        String key = null;
		ICommandLine cli = config.getSubcommandCommandLine();
		if (cli.hasOption(OPT_KEY)) {
			key = cli.getOption(OPT_KEY);
		}
		
		Map<String, String> map = new TreeMap<String, String>();
		if (printValue(key, PreferenceRegistry.PREF_CONTENT_TRANSFER_THREADS)){
			map.put(PreferenceRegistry.PREF_CONTENT_TRANSFER_THREADS,
					Integer.toString(SCMPlatform.getMaxContentThreads()));
		}
		if (printValue(key, PreferenceRegistry.PREF_REPO_CONNECTION_TIMEOUT)) {
            map.put(PreferenceRegistry.PREF_REPO_CONNECTION_TIMEOUT,
                    Integer.toString(repo.getConnectionTimeout()));
		}
		
		String[] strings = new String[map.size() * 3];
		int i = 0;
		for (Map.Entry<String, String> entry : map.entrySet()) {
			strings[i++] = entry.getKey();
            strings[i++] = "-";
			strings[i++] = entry.getValue();
		}
		StringUtil.printTable(new IndentingPrintStream(config.getContext().stdout()), 3, true, strings);
	}
	private static boolean printValue(String str, String key) {
		if (str == null) {
			return true;
		}
		return str.equalsIgnoreCase(key);
	}
}
