/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.filesystem.cli.tools.verify;
        
import static com.ibm.team.filesystem.cli.tools.verify.EclipseWorkspaceCompareCmdOptions.OPT_EXCLUDES;

import java.util.HashSet;
import java.util.List;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;

import com.ibm.team.filesystem.client.FileSystemException;
import com.ibm.team.filesystem.client.IShareable;
import com.ibm.team.filesystem.client.ISharingDescriptor;
import com.ibm.team.filesystem.client.internal.Shareable;
import com.ibm.team.filesystem.cli.core.AbstractSubcommand;
import com.ibm.team.filesystem.cli.core.util.StatusHelper;

public class EclipseWorkspaceCompareCmd extends AbstractSubcommand {
	public EclipseWorkspaceCompareCmd() {}
	
	public void run() throws FileSystemException {
		// Consume args
		HashSet<String> excludes = new HashSet<String>();
		{
			List<String> toExclude = config.getSubcommandCommandLine().getOptions(OPT_EXCLUDES);

			excludes.addAll(toExclude);
		}
		
		// Refresh resource tree
		try {
			ResourcesPlugin.getWorkspace().getRoot().refreshLocal(IResource.DEPTH_INFINITE, null);
		} catch (CoreException e) {
			throw StatusHelper.failure("Refresh failed", e);
		}
		
		
		// Verify projects
		for (IProject project : ResourcesPlugin.getWorkspace().getRoot().getProjects()) {
			// Verify each project separately.
			Shareable shareable = (Shareable)project.getAdapter(IShareable.class);
			ISharingDescriptor desc = shareable.getShare(null).getSharingDescriptor();
			
			try {
				new FileSystemComparator(shareable).assertEquals(desc, excludes, config.getRepositoryURI(desc.getRepositoryId(), null));
			} catch (Exception e) {
				throw StatusHelper.failure("Compare failed", e);
			}
		}
	}
}
