/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.filesystem.cli.tools.verify;

import java.io.IOException;
import java.io.OutputStreamWriter;

import com.ibm.team.filesystem.client.FileSystemException;
import com.ibm.team.filesystem.client.internal.MetronomeEventModel;
import com.ibm.team.filesystem.client.internal.utils.IFSDaemon;
import com.ibm.team.filesystem.client.rest.IDebugRestClient;
import com.ibm.team.filesystem.client.restproxy.DaemonRegistry;
import com.ibm.team.filesystem.client.restproxy.Discovery2;
import com.ibm.team.filesystem.common.internal.rest.debug.core.MetronomeRootDTO;
import com.ibm.team.rtc.cli.infrastructure.internal.core.IOptionSource;
import com.ibm.team.rtc.cli.infrastructure.internal.parser.ICommandLine;
import com.ibm.team.rtc.cli.infrastructure.internal.parser.Options;
import com.ibm.team.rtc.cli.infrastructure.internal.parser.PositionalOptionDefinition;
import com.ibm.team.rtc.cli.infrastructure.internal.parser.exceptions.ConflictingOptionException;
import com.ibm.team.filesystem.cli.core.AbstractSubcommand;
import com.ibm.team.filesystem.cli.core.util.StatusHelper;

public class MetronomeCmd extends AbstractSubcommand implements IOptionSource {
	
	private static final PositionalOptionDefinition OPT_PORT = new PositionalOptionDefinition("port",  1, 1);
	
	public Options getOptions() throws ConflictingOptionException {
		Options opts = new Options(false);
		
		opts.addOption(OPT_PORT, "Port of the daemon to query");
	        
		return opts;
	}


	public void run() throws FileSystemException {
        IFSDaemon daemon = (IFSDaemon)config.getContext().getRemoteObject();
        if (daemon != null) {
        	throw StatusHelper.internalError("You shouldn't be able to run this in a lightweight client.");
        }
        
        // Get the args
        int port;
        {
        	ICommandLine subcli = config.getSubcommandCommandLine();
        	String portString = subcli.getOption(OPT_PORT).trim();
        	port = Integer.parseInt(portString);
        	if (port < 1) {
        		throw StatusHelper.argSyntax("Port must be a positive integer");
        	}
        }
        
        DaemonRegistry registry = new DaemonRegistry(config.getConfigDirectory());
        
        // Connect
        Discovery2.DiscoveryParams discoverParams = new Discovery2.DiscoveryParams()
                .setDaemonRegistry(registry)
                .setInterface(IDebugRestClient.class)
                .setRunInProcess(false)
                .setAllowStart(false)
                .setPort(port);
        
        IDebugRestClient client = Discovery2.RESOLVER.resolve(discoverParams, null);
        
        // Dump the stats
        MetronomeRootDTO stats;
		try {
			stats = client.getMetronomeStatistics();
		} catch (IllegalStateException e) {
			throw StatusHelper.disallowed("Target daemon must be running with " + MetronomeEventModel.ENV_VAR + " set");
		}
		
        OutputStreamWriter writer = new OutputStreamWriter(config.getContext().stdout());
        try {
        	stats.tabulate(writer);
        	writer.flush();
        } catch (IOException e) {
        	throw StatusHelper.internalError("Couldn't tabulate result", e);
        }
        
        // We don't close the output stream writer because we don't want to 
        // inadvertently close our stdout. 
	}
}
