/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.rtc.cli.infrastructure.internal;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.RegistryFactory;
import org.eclipse.equinox.app.IApplication;
import org.eclipse.equinox.app.IApplicationContext;

import com.ibm.team.rtc.cli.infrastructure.internal.core.Constants;
import com.ibm.team.rtc.cli.infrastructure.internal.core.LocalContext;
import com.ibm.team.rtc.cli.infrastructure.internal.core.SubcommandLauncher;
import com.ibm.team.rtc.cli.infrastructure.internal.util.StringUtil;

/**
 * 
 */
public class Application implements IApplication {
	private static final Log log = LogFactory.getLog(Application.class.getName());

	public Object start(IApplicationContext context) throws Exception {
		final long start = System.currentTimeMillis();
		
		String[] args = (String[])context.getArguments().get("application.args"); //$NON-NLS-1$
		
		LocalContext ctx = new LocalContext(start, getAppName(context), System.out, System.err, System.in, 
				args, System.getenv(), StringUtil.findEffectiveCwd(args));
		
		SubcommandLauncher launcher = new SubcommandLauncher(Constants.PT_ATTR_SCOPE_DEFAULT, ctx.stderr());
		
		return launcher.run(ctx, context, new NullProgressMonitor());
	}

	private String getAppName(IApplicationContext context) {
		String appName = context.getBrandingName();
		
		IConfigurationElement[] extensions = RegistryFactory.getRegistry().getConfigurationElementsFor(Activator.PLUGIN_ID, Constants.PT_CUSTOMIZE_PRODUCT);
		if (extensions.length == 1) {
			appName = extensions[0].getAttribute(Constants.PT_ATTR_PRODUCT_NAME);
		} else if (extensions.length > 1) {
			log.warn("Multiple extensions found for extension point :" + Activator.PLUGIN_ID + "." + Constants.PT_CUSTOMIZE_PRODUCT); //$NON-NLS-1$ //$NON-NLS-2$
		}
		
		return appName;
	}

	public void stop() {
	}
}
