/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.rtc.cli.infrastructure.internal.core;

import org.eclipse.core.runtime.IStatus;

/**
 * Expected errors encountered during execution of the CLI. The status code 
 * carried in the status object is used as the return code for the app, and 
 * the message in the status is displayed to the user. 
 */
public class CLIClientException extends Throwable {    
    IStatus status;

	/** Appease the workbench */
	private static final long serialVersionUID = -2944859849671801502L;

	public CLIClientException() {
        this.status = null;
    }
	
	public CLIClientException(String message) {
	    super(message);
	}
	
	public CLIClientException(String message, Throwable cause) {
        super(message, cause);
    }
	
	public CLIClientException(Throwable cause) {
	    this(CLIClientException.getErrorMessage(cause), cause);
	}
	
	public CLIClientException(IStatus status) {
	    this.status = status;
	}
	
    public CLIClientException(IStatus status, Throwable cause) {
        this(CLIClientException.getErrorMessage(cause), cause);
        this.status = status;
    }

    private static String getErrorMessage(Throwable cause) {
        if (cause == null)
            return null;
        String message = cause.getMessage();
        if (message == null) {
            message = "Internal error"; //$NON-NLS-1$
        }
        return message;
    }

    public IStatus getStatus() {
	    return status;
	}
	
	public void setStatus(IStatus status) {
	    this.status = status;
	}
	
	public boolean showErrorMessage() {
        return (getStatus() != null && getStatus().getMessage() != null &&
                getStatus().getMessage().length() > 0) ?
                true : false;
    }
}
