/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.rtc.cli.infrastructure.internal.core;

import java.io.File;


import java.util.Collection;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.core.runtime.Platform;
import org.eclipse.equinox.app.IApplicationContext;

import com.ibm.team.rtc.cli.infrastructure.internal.core.IExecutionContext;
import com.ibm.team.rtc.cli.infrastructure.internal.Activator;
import com.ibm.team.rtc.cli.infrastructure.internal.core.IApplicationEndListener;
import com.ibm.team.rtc.cli.infrastructure.internal.core.IClientConfiguration;
import com.ibm.team.rtc.cli.infrastructure.internal.core.ISubcommandDefinition;
import com.ibm.team.rtc.cli.infrastructure.internal.parser.ICommandLine;

public class ClientConfiguration implements IClientConfiguration {
    
    protected SubcommandLoader loader;

    protected IExecutionContext environment;

    @Deprecated
    protected IApplicationContext context;

    protected ICommandLine args;

    protected List<ISubcommandDefinition> parents;

    protected ICommandLine subargs;

    protected File cwd;

    protected List<IApplicationEndListener> endListeners;

    /** The subcommand definition of the currently running subcommand. */
    protected ISubcommandDefinition defn;

    public void initialize(SubcommandLoader loader, IExecutionContext environment,
            IApplicationContext context, ISubcommandDefinition defn,
            List<ISubcommandDefinition> parents, ICommandLine args, ICommandLine subargs)
            throws CLIClientException {
        this.loader = loader;
        this.environment = environment;
        
        cwd = new File(environment.getCurrentWorkingDirectory());

        this.context = context;

        this.defn = defn;

        this.args = args;
        this.subargs = subargs;

        this.parents = parents;

        this.endListeners = new LinkedList<IApplicationEndListener>();

    }

    public File getCurrentWorkingDirectory() {
        return cwd;
    }
    
    public SubcommandLoader getSubcommandLoader() {
        return loader;
    }

    public Collection<ISubcommandDefinition> getSubcommandDefinitions() {
        return loader.getAllDefinitions();
    }

    public ISubcommandDefinition getSubcommandDefinition() {
        return defn;
    }

    public IApplicationContext getApplicationContext() {
        return context;
    }

    public ICommandLine getSubcommandCommandLine() {
        return subargs;
    }

    public ICommandLine getCommandLine() {
        return args;
    }

    public List<ISubcommandDefinition> getParents() {
        return parents;
    }
    
    public boolean mayInteract() {
        return false;
    }

    public void addApplicationEndListener(IApplicationEndListener listener) {
        endListeners.add(listener);
    }

    public void deInitialize() {
        for (IApplicationEndListener listener : endListeners) {
            listener.run(this);
        }
    }

    public IClientConfiguration getClientConfiguration() {
        return this;
    }

    public IExecutionContext getContext() {
        return environment;
    }

    public boolean isDryRun() {
        return (args == null || args.hasOption(SubcommandLauncher.OPT_DRY_RUN));
    }
}
