/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.rtc.cli.infrastructure.internal.core;

import org.eclipse.equinox.app.IApplication;

/**
 * Constants used by the CLI.
 */
public class Constants {

    /**
     * Set to <code>true</code> when running on MS Windows, <code>false</code>
     * otherwise.
     */
    public final static boolean ON_WINDOWS = System
            .getProperty("os.name").toLowerCase().contains("windows"); //$NON-NLS-1$//$NON-NLS-2$

    /**
     * Indicate that the app ran properly.
     */
    public final static Integer STATUS_OK = IApplication.EXIT_OK;

    /**
     * Indicates that there was a syntax error on the command-line. The user has
     * provided the wrong number of required arguments, an argument that the
     * given subcommand doesn't understand, or an argument that doesn't make
     * sense in the given context.
     * 
     * <p>
     * The user should modify the arguments and re-run.
     * 
     * @core
     */
    public final static int STATUS_ARGUMENT_SYNTAX_ERROR = 1;

    /**
     * Indicates that a failure occurred for an unknown reason.
     * 
     * @core
     */
    public static final int STATUS_FAILURE = 3;
    
    /** An internal error indicating that the client was not properly 
     * configured, specifically a subcommand couldn't be loaded, likely due 
     * to a configuration error by the programmer.
     * 
     * @core
     */
    public static final int STATUS_CONFIG_FAILURE = 4;

    /** An error of last resort. Indicates that condition that should have 
     * been handled was not handled. A more severe version of 
     * {@link #STATUS_FAILURE}.
     * 
     *  @core
     */
    public static final int STATUS_INTERNAL_ERROR = 7;

    /** Indicates that the requested subcommand is not known to the CLI. The 
     * user should run the "help" subcommand to find determine appropriate 
     * argument. 
     * 
     * @core
     */
    public static final int STATUS_UNKNOWN_SUBCOMMAND = 12;

    // NOTE! STATUS 13 IS USED BY OSGI TO INDICATE AN EXCEPTION. DO NOT USE.
    
    /** 
     * Test whether or not a stack trace should be shown for the given error
     * state. 
     */
    public static boolean showStackTraceFor(int code) {
        return code == STATUS_INTERNAL_ERROR || code == STATUS_FAILURE;
    }

    /** Name of the extension point used to provide the client configuration. */
    public static final String PT_CLIENTCONFIG = "clientconfiguration"; //$NON-NLS-1$

    /** Name of the extension point used to add subcommands. */
    public static final String PT_SUBCOMMAND = "subcommand"; //$NON-NLS-1$

    /** Name of the "name" attribute. Human-meaningful name of subcommands. */
    public static final String PT_ATTR_NAME = "name"; //$NON-NLS-1$

    /**
     * Name of the attribute providing a short, human-meaningful explanation of
     * the subcommand.
     */
    public static final String PT_ATTR_SHORT_HELP = "shortHelp"; //$NON-NLS-1$

    /**
     * Name of the attribute providing a long, human meaningful explanation of
     * subcommand.
     */
    public static final String PT_ATTR_LONG_HELP = "longHelp"; //$NON-NLS-1$

    /** Name of the class providing the behaviour for the given subcommand. */
    public static final String PT_ATTR_CLASS = "class"; //$NON-NLS-1$

    /** Name of the Eclipse workspace initialization policy. */
    public static final String PT_ATTR_INIT_POLICY = "initpolicy"; //$NON-NLS-1$

    /** Name of the class providing the options for a subcommand. */
    public static final String PT_ATTR_OPTIONS = "options"; //$NON-NLS-1$

    /** Alternate names for the subcommand. Space separated list. */
    public static final String PT_ATTR_ALIASES = "aliases"; //$NON-NLS-1$

    /** Unique ID for the subcommand. */
    public static final String PT_ATTR_ID = "id"; //$NON-NLS-1$

    /**
     * The ID of the parent.
     */
    public static final String PT_ATTR_PARENT = "parent"; //$NON-NLS-1$

    /**
     * Needed for ISubcommandDefinition#showInHelpListing()
     * 
     * @deprecated
     */
    public static final String PT_ATTR_HIDE = "hideFromHelp"; //$NON-NLS-1$

    /**
     * A grouping for the subcommand.
     */
    public static final String PT_ATTR_SCOPE = "scope"; //$NON-NLS-1$

    /**
     * The "default" group that subcommands can fall into.
     * 
     * @see #PT_ATTR_SCOPE
     */
    public static final String PT_ATTR_SCOPE_DEFAULT = "default"; //$NON-NLS-1$

    /** Override value  */
    public static final String PT_ATTR_OVERRIDE = "override"; //$NON-NLS-1$
    
    /** Indicates whether the specified command is deprecated  */
    public static final String PT_ATTR_DEPRECATED = "deprecated"; //$NON-NLS-1$

    /**
     * Name of the extension point used to provide information for customizing
     * the product.
     */
    public static final String PT_CUSTOMIZE_PRODUCT = "customizeProduct"; //$NON-NLS-1$
    public static final String PT_ATTR_PRODUCT_NAME = "appName"; //$NON-NLS-1$

    
    public static final String JAZZSCM_FORCE_GLOBBING = "JAZZSCM_FORCE_GLOBBING"; //$NON-NLS-1$
    public static final String JAZZSCM_FORCE_GLOB_VALUE = "1"; //$NON-NLS-1$
    
    public static final String SHOW_DEPRECATED_HELP = "SHOW_DEPRECATED_HELP";
    public static final String SHOW_DEPRECATED_HELP_VALUE = "1"; //$NON-NLS-1$
}
