/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.rtc.cli.infrastructure.internal.core;

import java.io.File;


import java.util.Collection;
import java.util.List;

import org.eclipse.core.runtime.Platform;
import org.eclipse.equinox.app.IApplicationContext;

import com.ibm.team.rtc.cli.infrastructure.internal.core.IExecutionContext;
import com.ibm.team.rtc.cli.infrastructure.internal.Activator;
import com.ibm.team.rtc.cli.infrastructure.internal.parser.ICommandLine;

/**
 * Provides access to the configuration of the client. Methods on this class
 * query configuration specified on the command-line, in the config files, or in
 * the environment.
 * 
 * <p/>
 * This interface should not be implemented by clients.
 */
public interface IClientConfiguration {
    
    public static final boolean DEBUG_TIMING = Activator.getDefault().isDebugging() &&
            "true".equalsIgnoreCase(Platform.getDebugOption(
                    "com.ibm.team.rtc.cli.infrastructure/debug/timing")); //$NON-NLS-1$ //$NON-NLS-2$
    
    public static final boolean DEBUG_OUTPUT = Activator.getDefault().isDebugging() &&
            "true".equalsIgnoreCase(Platform.getDebugOption(
                    "com.ibm.team.rtc.cli.infrastructure/debug/output")); //$NON-NLS-1$ //$NON-NLS-2$
    
    public static final String PROP_SHOW_ERRORS = "cli.show.own.errors"; //$NON-NLS-1$
    
    // do not use - retained for backwards compatibility after moving the code from scm component 
    // to a generic component. Use PROP_SHOW_ERRORS.
    public static final String SCM_PROP_SHOW_ERRORS = "scm.show.own.errors"; //$NON-NLS-1$
    
    /** Initialize the configuration */
    public void initialize(SubcommandLoader loader, IExecutionContext environment,
            IApplicationContext context, ISubcommandDefinition defn,
            List<ISubcommandDefinition> parents, ICommandLine args, ICommandLine subargs)
                    throws CLIClientException;
    
    /** Return the subcommand loader */
    public SubcommandLoader getSubcommandLoader();

    /** Return a list of all subcommands available in this client. */
    public Collection<ISubcommandDefinition> getSubcommandDefinitions();

    /** Return the subcommand definition of the currently executing subcommand. */
    public ISubcommandDefinition getSubcommandDefinition();

    /** Get the command-line for the subcommand. */
    public ICommandLine getSubcommandCommandLine();

    /** Get the command-line issued to the app. */
    public ICommandLine getCommandLine();

    /**
     * Get the directory we are running in.
     * 
     * @return <code>null</code> if the CWD has not been set, some directory
     *         other wise.
     * 
     * @deprecated {@link IExecutionContext#getCurrentWorkingDirectory()}.
     */
    public File getCurrentWorkingDirectory();

    /**
     * Expose the underlying OSGI-ness.
     * 
     * @deprecated This will likely be deleted in the 2.x+ timeframe.
     */
    public IApplicationContext getApplicationContext();
    
    /**
     * @return <code>true</code> if the client may interactively query the 
     *      user, <code>false</code> otherwise. 
     */
    public boolean mayInteract();    

    /**
     * Adds a listener to execute when the application ends.
     * 
     * @param listener
     *            The listener to add.
     */
    public void addApplicationEndListener(IApplicationEndListener listener);

    /**
     * Notify the receiver that it's life cycle is at an end. Runs the
     * {@link IApplicationEndListener}s and cleans up anything that needs a
     * cleaning.
     */
    public void deInitialize();
    
    /**
     * @return Return whatever parent subcommands may have appeared before this
     *         one on the commandline.
     */
    public List<ISubcommandDefinition> getParents();

    /**
     * Get the streams/environment that the subcommand can use to interact with
     * the user.
     */
    public IExecutionContext getContext();
    
    /**
     * @return <code>true</code> if the client should not modify anything on 
     *      disk or in the repository, <code>false</code> if the user may 
     *      modify anything. 
     */
    public boolean isDryRun();
}
