/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.rtc.cli.infrastructure.internal.core;

import java.io.IOException;
import java.io.InputStream;
import java.io.PrintStream;
import java.util.Map;

/**
 * Allows a subcommand to interact with the user.
 */
public interface IExecutionContext {

    /** Get a stream representing stdout. */
    public PrintStream stdout();

    /** Get a stream representing stderr. */
    public PrintStream stderr();

    /**
     * Get a stream representing the standard in.
     */
    public InputStream stdin();

    /**
     * Return the arguments passed in on the command line. Note that these are
     * platform-specific: on *nix, wildcard expansion will have occurred,
     * whereas Windows will still need expansion.
     */
    public String[] arguments();

    /**
     * Return the name of the app.
     */
    public String getAppName();

    /**
     * Get the environment variables.
     * 
     * <p>
     * Note: When possible, take values from {@link IClientConfiguration} before
     * using the remote environment variables. For example, the current working
     * directory is available on
     * {@link IClientConfiguration#getCurrentWorkingDirectory()}.
     */
    public Map<String, String> environment();

    /**
     * Set the error message to display on termination. This is legacy support
     * for the OSGi-based launcher.
     */
    public void setTerminatingError(String title, String message);

    /**
     * Return the canonical working directory (aka CWD, PWD). If "-d" is
     * specified on the command line, its value is returned.
     * 
     * @see CommonOptions#OPT_DIRECTORY
     * 
     * @return A string representation of the current working directory. Never
     *         null.
     */
    public String getCurrentWorkingDirectory();

    /** Read an input from the user, with the given prompt. 
     */
    public String readInput(String prompt, boolean mask) throws IOException;

    /**
     * @return The time the subcommand started being executed.
     */
    public long getStartTime();
    
    /**
     * @return The context whether it is local or remote
     */
    public boolean isLocal();

    /**
     * @return Return the remote object if it is a remote context
     */
    public Object getRemoteObject();
}
