/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.rtc.cli.infrastructure.internal.core;

import java.io.IOException;
import java.io.InputStream;
import java.io.PrintStream;
import java.util.Map;

import com.ibm.team.rtc.cli.infrastructure.internal.util.PromptUtil;

/**
 * In interaction context that wraps the current environment.
 */
public class LocalContext extends AbstractExecutionContext {

    private final String appName;
    private final PrintStream stdout;
    private final PrintStream stderr;
    private final InputStream stdin;
    private final String[] arguments;
    private final Map<String, String> env;
    private final String cwd;
    private final boolean local;

    public LocalContext(String appName, PrintStream stdout, PrintStream stderr, InputStream stdin,
            String[] arguments, Map<String, String> env, String cwd) {
        this(System.currentTimeMillis(), appName, stdout, stderr, stdin, arguments, env, cwd);
    }

    public LocalContext(long start, String appName, PrintStream stdout, PrintStream stderr,
            InputStream stdin, String[] arguments, Map<String, String> env, String cwd) {
        super(start);
        this.appName = appName;
        this.stdout = stdout;
        this.stderr = stderr;
        this.stdin = stdin;
        this.arguments = arguments;
        this.env = env;
        this.cwd = cwd;
        this.local = true;
    }

    public PrintStream stderr() {
        return stderr;
    }

    public PrintStream stdout() {
        return stdout;
    }

    public InputStream stdin() {
        return stdin;
    }

    public String[] arguments() {
        return arguments;
    }

    public String getAppName() {
        return appName;
    }

    public void setTerminatingError(String title, String message) {
        System.getProperties().setProperty(
                "eclipse.exitdata", "<title>" + title + "</title>" + message); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
    }

    public Map<String, String> environment() {
        return env;
    }

    public String getCurrentWorkingDirectory() {
        return cwd;
    }
    
    public String readInput(String prompt, boolean mask) throws IOException {
        return PromptUtil.prompt(prompt, mask, stdin, stdout);
    }

    public boolean isLocal() {
        return local;
    }
    
    public Object getRemoteObject() {
        return null;
    }
}
