/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.rtc.cli.infrastructure.internal.parser;

import com.ibm.team.rtc.cli.infrastructure.internal.Messages;

/**
 * 
 */
public class AbstractOptionDefinition extends OptionKey implements IOptionDefinition {

    private final IOptionKey id;

    protected final static IOptionKey USE_SELF_AS_ID = new IOptionKey() {
        public String getName() { return null; }
        public String getDelimiter() { return null; }
    };

    public AbstractOptionDefinition(IOptionKey id, String name) {
        super(name);

        if (id == null) {
            throw new IllegalArgumentException(Messages.AbstractOptionDefinition_0);
        }

        if (name == null) {
            throw new IllegalArgumentException(Messages.AbstractOptionDefinition_1);
        }

        this.id = (id == USE_SELF_AS_ID ? this : id);
    }

    public AbstractOptionDefinition(IOptionKey id, String name, String delimiter) {
        super(name, delimiter);

        if (id == null) {
            throw new IllegalArgumentException(Messages.AbstractOptionDefinition_0);
        }

        if (name == null) {
            throw new IllegalArgumentException(Messages.AbstractOptionDefinition_1);
        }

        this.id = (id == USE_SELF_AS_ID ? this : id);
    }

    public AbstractOptionDefinition(String name) {
        this(USE_SELF_AS_ID, name, null);
    }

    final public IOptionKey getId() {
        return id;
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.lang.Object#toString()
     */
    @Override
    public String toString() {
        return getName();
    }

    public void matchArgument(IParseController controller) {
    }
    
    public boolean isRequired() {
    	return true;
    }
}
