/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.rtc.cli.infrastructure.internal.parser;


public class CommandLineArgument implements ICommandLineArgument {

    protected IOptionKey key;
    protected String arg;
    protected String argPrefix;
    protected String argSuffix;

    /**
     * @param arg   Argument entered in the command line.
     */
    public CommandLineArgument(IOptionKey optionKey, String arg) {
        // this should never happen as checks are made in the parsing code
        assert arg != null;

        this.key = optionKey;
        this.arg = arg;
    }

    /**
     * @param arg       Argument entered in the command line.
     * @param prefix    If a token was used to split the argument, provide the prefix.
     * @param suffix    If a token was used to split the argument, provide the suffix.
     */
    public CommandLineArgument(IOptionKey optionKey, String arg, String prefix, String suffix) {
        // this should never happen as checks are made in the parsing code
        assert arg != null;

        this.key = optionKey;
        this.arg = arg;
        this.argPrefix = prefix;
        this.argSuffix = suffix;
    }
    
    public IOptionKey getKey() {
        return key;
    }

    public String getValue() {
        return arg;
    }

    public String getValuePrefix() {
        return argPrefix;
    }

    public String getValueSuffix() {
        return argSuffix;
    }
}
