/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.rtc.cli.infrastructure.internal.parser;

import java.util.Collection;
import java.util.LinkedList;
import java.util.List;

import com.ibm.team.rtc.cli.infrastructure.internal.util.Pair;

public class ContinuousGroup implements IOptionGroup {
	private final LinkedList<IOptionGroup> groups  = new LinkedList<IOptionGroup>();
	private boolean isRequired = false;
	
	public ContinuousGroup() {		
	}
	
	public ContinuousGroup(boolean isRequired) {
		this.isRequired = isRequired;
	}
	
	public ContinuousGroup addOption(INamedOptionDefinition opt, String helpString, boolean isRequired) {
		return addOptionHelper(opt, helpString, isRequired);
	}
	
	public ContinuousGroup addOption(IPositionalOptionDefinition opt, String helpString, boolean isRequired) {
        return addOptionHelper(opt, helpString, isRequired);
    }
	
	public ContinuousGroup addOption(IOptionKey optKey, String shortOpt, String longOpt, String helpString, int argCount, boolean isRequired) {
		INamedOptionDefinition opt = new NamedOptionDefinition(optKey, shortOpt, longOpt, argCount);
		return addOptionHelper(opt, helpString, isRequired);
	}
	
	public ContinuousGroup addOption(SimpleGroup group) {
		groups.add(group);
		return this;
	}
	
	public ContinuousGroup addOption(MutuallyExclusiveGroup group) {
		groups.add(group);
		return this;
	}
	
	private ContinuousGroup addOptionHelper(INamedOptionDefinition opt, String helpString, boolean isRequired) {
		SimpleGroup grp = new SimpleGroup(isRequired);
		grp.addOption(opt, helpString, isRequired);
		groups.add(grp);
		return this;
	}
	
    private ContinuousGroup addOptionHelper(IPositionalOptionDefinition opt, String helpString, boolean isRequired) {
        SimpleGroup grp = new SimpleGroup(isRequired);
        grp.addOption(opt, helpString);
        groups.add(grp);
        return this;
    }
    
    public Collection<IOptionGroup> getOptions() {
        return groups;
    }

    public Collection<SimpleGroup> getSimpleGroups() {
        LinkedList<SimpleGroup> simpleGroups = new LinkedList<SimpleGroup>();
        for (IOptionGroup grp : groups) {
            if (grp instanceof SimpleGroup) {
                simpleGroups.add((SimpleGroup) grp);
            }
        }
		return simpleGroups;
	}
	
	public Collection<MutuallyExclusiveGroup> getMutuallyExclusiveGroups() {
        LinkedList<MutuallyExclusiveGroup> mutuallyExclusiveGroups = new LinkedList<MutuallyExclusiveGroup>();
        for (IOptionGroup grp : groups) {
            if (grp instanceof MutuallyExclusiveGroup) {
                mutuallyExclusiveGroups.add((MutuallyExclusiveGroup) grp);
            }
        }
		return mutuallyExclusiveGroups;
	}

	@Override
	public Collection<Pair<INamedOptionDefinition, String>> getNamedOptionsWithHelp() {
		List<Pair<INamedOptionDefinition, String>> flattenedNamedOptions = new LinkedList<Pair<INamedOptionDefinition, String>>();
        for (IOptionGroup grp : groups) {
            flattenedNamedOptions.addAll(grp.getNamedOptionsWithHelp());
        }
		return flattenedNamedOptions;	
	}
	
	@Override
	public Collection<INamedOptionDefinition> getNamedOptions() {
		List<INamedOptionDefinition> flattenedNamedOptions = new LinkedList<INamedOptionDefinition>();
        for (IOptionGroup grp : groups) {
            flattenedNamedOptions.addAll(grp.getNamedOptions());
        }
		return flattenedNamedOptions;
	}
	
	@Override
	public Collection<IPositionalOptionDefinition> getPositionalOptions() {
		List<IPositionalOptionDefinition> posOptions = new LinkedList<IPositionalOptionDefinition>();
        for (IOptionGroup grp : groups) {
            posOptions.addAll(grp.getPositionalOptions());
        }
		return posOptions;
	}
	
	/***
	 * By default, this will be false and the required-ness of the group is determined by its members
     * However if the user explicitly makes the group required, then this group is mandatory irrespective
     * of whether any of its member is required or not.
	 * @return true if the ContinuousGroup is required, false otherwise
	 */
	@Override
	public boolean isRequired() {
		return isRequired;
	}

	@Override
	public Collection<Pair<IPositionalOptionDefinition, String>> getPositionalOptionsWithHelp() {
	    List<Pair<IPositionalOptionDefinition, String>> posOptions = new LinkedList<Pair<IPositionalOptionDefinition, String>>();
        for (IOptionGroup grp : groups) {
            posOptions.addAll(grp.getPositionalOptionsWithHelp());
        }
        return posOptions;
	}
}
