/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.rtc.cli.infrastructure.internal.parser;

import java.util.Collection;
import com.ibm.team.rtc.cli.infrastructure.internal.util.Pair;

/// Design notes
/// Currently all implementations expose some non standard methods to get 
/// their members. For instance, ContinuousGroup exposes two methods - one for 
/// SimpleGroup and other for MutallyExclusiveGroup. The problem with this approach
/// is that for printing, we print mutuallyexclusiveGroup followed by simpleGroup.
/// This breaks the assumption of the developer that options will be printed in the 
/// order in which they were added. A better option here could be to expose 
/// a standard interface member getOptionsForPrint that could return Objects[].
/// We then use an enumerator logic (like the one in getShortUsageHelp()) to look
/// at the type of each of the object and calls the appropriate printer. 
///
public interface IOptionGroup {
	
	Collection<Pair<INamedOptionDefinition, String>> getNamedOptionsWithHelp();
	
	Collection<INamedOptionDefinition> getNamedOptions();
	
	Collection<IPositionalOptionDefinition> getPositionalOptions();
	
	Collection<Pair<IPositionalOptionDefinition, String>> getPositionalOptionsWithHelp();
	
	boolean isRequired();

}
