/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.rtc.cli.infrastructure.internal.parser;

/**
 * An option associated with a position on the command-line.
 */
public interface IPositionalOptionDefinition extends IOptionDefinition {
    /**
     * Gets the minimum number of arguments this option expects. Must be greater
     * than zero.
     */
    public int getMinCount();

    /**
     * Gets the maximum number of arguments this option expects. May not be
     * smaller than {@link #getMinCount()}, or equal to
     * {@link Options#ARG_INFINITE}.
     */
    public int getMaxCount();
    
    public boolean isRequired();
    
    /**
     * Even though the option takes zero arguments we may want to show it
     * as a required argument. This is required when the command has mutually
     * exclusive group of options and this option shows up in only one group.
     * In this case, we have to handle the required check in the command itself. 
     */
    public void setShowAsRequired();
    
    public boolean isShowAsRequired();
}
