/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.rtc.cli.infrastructure.internal.parser;

import java.util.Collection;
import java.util.LinkedList;
import java.util.List;
import com.ibm.team.rtc.cli.infrastructure.internal.util.Pair;

public class MutuallyExclusiveGroup implements IOptionGroup{
	private LinkedList<SimpleGroup> grps = new LinkedList<SimpleGroup>();
	private boolean isRequired = false;

	public MutuallyExclusiveGroup() {
		
	}

	public MutuallyExclusiveGroup(boolean isRequired) {
		this.isRequired = isRequired;
	}
	
	public MutuallyExclusiveGroup addOption(INamedOptionDefinition opt, String helpString, boolean isRequired) {
		return addOptionHelper(opt, helpString, isRequired);
	}
	
	public MutuallyExclusiveGroup addOption(IOptionKey optKey, String shortOpt, String longOpt, String helpString, int argCount, boolean isRequired) {
		INamedOptionDefinition opt = new NamedOptionDefinition(optKey, shortOpt, longOpt, argCount);
		return addOptionHelper(opt, helpString, isRequired);
	}
	
	private MutuallyExclusiveGroup addOptionHelper(INamedOptionDefinition opt, String helpString,boolean isRequired) {
		// For a simple group with only one option, the requiredness of the group is determined by the requiredness of its single option
		SimpleGroup grp = new SimpleGroup(isRequired);
		grp.addOption(opt, helpString, isRequired);
		grps.add(grp);
		return this;
	}
	
	public MutuallyExclusiveGroup addOption(IPositionalOptionDefinition opt, String helpString) {
		SimpleGroup grp = new SimpleGroup(opt.isRequired());
		grp.addOption(opt, helpString);
		grps.add(grp);
		return this;
	}

	public MutuallyExclusiveGroup addOption(SimpleGroup grp) {
		grps.add(grp);
		return this;
	}
	
	public Collection<SimpleGroup> getSimpleGroups() {
		return grps;
	}

	@Override
	public Collection<Pair<INamedOptionDefinition, String>> getNamedOptionsWithHelp() {
		List<Pair<INamedOptionDefinition,String>> flattedNamedOptions = new LinkedList<Pair<INamedOptionDefinition, String>>();
		for (SimpleGroup grp : grps) {
			flattedNamedOptions.addAll(grp.getNamedOptionsWithHelp());
		}
		return flattedNamedOptions;	
	}
	
	@Override
	public Collection<INamedOptionDefinition> getNamedOptions() {
		List<INamedOptionDefinition> flattenedNamedOptions = new LinkedList<INamedOptionDefinition>();
		for (SimpleGroup grp : grps) {
			flattenedNamedOptions.addAll(grp.getNamedOptions());
		}
		return flattenedNamedOptions;
	}

	@Override
	public  Collection<IPositionalOptionDefinition> getPositionalOptions() {
		List<IPositionalOptionDefinition> flattenedPositionalOptions = new LinkedList<IPositionalOptionDefinition>();
		for (SimpleGroup grp : grps) {
			flattenedPositionalOptions.addAll(grp.getPositionalOptions());
		}
		return flattenedPositionalOptions;
	}
	
	@Override
	public Collection<Pair<IPositionalOptionDefinition, String>> getPositionalOptionsWithHelp() {
		List<Pair<IPositionalOptionDefinition, String>> flattenedPositionalOptionsWithHelp = new LinkedList<Pair<IPositionalOptionDefinition, String>>();
		for (SimpleGroup grp : grps) {
			flattenedPositionalOptionsWithHelp.addAll(grp.getPositionalOptionsWithHelp());
		}
		return flattenedPositionalOptionsWithHelp;
	}

	/***
	 * By default, this will be false and the required-ness of the group is determined by its members
     * However if the user explicitly makes the group required, then this group is mandatory irrespective
     * of whether any of its member is required or not.
	 * @return true if the MutuallyExclusiveGroup is required, false otherwise
	 */
	@Override
	public boolean isRequired() {
		return isRequired;
	}
}
