/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.rtc.cli.infrastructure.internal.parser;

import org.eclipse.osgi.util.NLS;

import com.ibm.team.rtc.cli.infrastructure.internal.Messages;

/**
 * 
 */
public class PositionalOptionDefinition extends AbstractOptionDefinition implements
        IPositionalOptionDefinition {

    final private int minCount;
    final private int maxCount;
    boolean showAsRequired = false;

    public PositionalOptionDefinition(IOptionKey id, String name, int minCount, int maxCount) {
        this(id, name, minCount, maxCount, null);
    }

    public PositionalOptionDefinition(IOptionKey id, String name, int minCount, int maxCount,
            String delimiter) {
        super(id, name, delimiter);

        if (minCount > maxCount && maxCount != Options.ARG_INFINITE) {
            throw new IllegalArgumentException(NLS.bind(Messages.PositionalOptionDefinition_0,
                    minCount, maxCount));
        }

        this.minCount = minCount;
        this.maxCount = maxCount;
    }

    public PositionalOptionDefinition(String name, int minCount, int maxCount) {
        this(USE_SELF_AS_ID, name, minCount, maxCount, null);
    }

    public PositionalOptionDefinition(String name, int minCount, int maxCount, String delimiter) {
        this(USE_SELF_AS_ID, name, minCount, maxCount, delimiter);
    }

    @Override
    public int getMaxCount() {
        return maxCount;
    }

    @Override
    public int getMinCount() {
        return minCount;
    }

    @Override
    public String toString() {
        return super.getName();
    }

	@Override
	public boolean isRequired() {
		return (getMinCount() > 0);
	}

    @Override
    public void setShowAsRequired() {
        showAsRequired = true;
    }

    @Override
    public boolean isShowAsRequired() {
        return showAsRequired;
    }
}
