/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.rtc.cli.infrastructure.internal.parser;

import java.util.Collection;
import java.util.LinkedList;
import java.util.List;

import com.ibm.team.rtc.cli.infrastructure.internal.util.Pair;

public class SimpleGroup implements IOptionGroup {
	private final LinkedList<NamedOptionWrapper> namedOpts = new LinkedList<NamedOptionWrapper>();
	private final LinkedList<PositionalDefinitionWrapper> positionalOpts = new LinkedList<PositionalDefinitionWrapper>();
	
	private boolean required = true;
	
	public SimpleGroup(boolean required) {
		this.required = required;
	}

	public SimpleGroup addOption(INamedOptionDefinition opt, String helpString, boolean isRequired) {
		namedOpts.add(new NamedOptionWrapper(opt, helpString, isRequired));
		return this;
	}
	
	public SimpleGroup addOption(IPositionalOptionDefinition positionalOpt, String helpString) {
		positionalOpts.add(new PositionalDefinitionWrapper(positionalOpt, helpString));
		return this;
	}
	
	public List<Pair<INamedOptionDefinition, Boolean>>  getNamedOptionsWithFlag() {
		List<Pair<INamedOptionDefinition, Boolean>> namedOptPairs = new LinkedList<Pair<INamedOptionDefinition, Boolean>>();
		for(NamedOptionWrapper namedOptWrapper : namedOpts)
		{
			namedOptPairs.add(new Pair<INamedOptionDefinition, Boolean>(namedOptWrapper.getNamedOpt(), namedOptWrapper.isRequired()));
		}
		return namedOptPairs;
	}
	
	@Override
	public Collection<Pair<INamedOptionDefinition,String>> getNamedOptionsWithHelp() {
		List<Pair<INamedOptionDefinition,String>> flattedNamedOptions = new LinkedList<Pair<INamedOptionDefinition, String>>();
		for(NamedOptionWrapper namedOptWrapper : namedOpts) {
			flattedNamedOptions.add(new Pair<INamedOptionDefinition, String>(namedOptWrapper.getNamedOpt(), namedOptWrapper.getHelpString()));
		}
		return flattedNamedOptions;
	}

	@Override
	public Collection<INamedOptionDefinition> getNamedOptions() {
		List<INamedOptionDefinition> flattedNamedOptions = new LinkedList<INamedOptionDefinition>();
		for(NamedOptionWrapper namedOptWrapper : namedOpts) {
			flattedNamedOptions.add(namedOptWrapper.getNamedOpt());
		}
		return flattedNamedOptions;
	}
	
	@Override
	public Collection<IPositionalOptionDefinition> getPositionalOptions() {
		List<IPositionalOptionDefinition> flattedPositionalOptions = new LinkedList<IPositionalOptionDefinition>();
		for(PositionalDefinitionWrapper positionalOptWrapper : positionalOpts) {
			flattedPositionalOptions.add(positionalOptWrapper.getPositionalOpt());
		}
		return flattedPositionalOptions;	
	}
	
	@Override
	public Collection<Pair<IPositionalOptionDefinition,String>> getPositionalOptionsWithHelp() {
		List<Pair<IPositionalOptionDefinition,String>> flattedPositionalOptions = new LinkedList<Pair<IPositionalOptionDefinition, String>>();
		for(PositionalDefinitionWrapper positionalOptWrapper : positionalOpts) {
			flattedPositionalOptions.add(new Pair<IPositionalOptionDefinition, String>(positionalOptWrapper.getPositionalOpt(), positionalOptWrapper.getHelpString()));
		}
		return flattedPositionalOptions;
	}
	
	@Override
	public boolean isRequired() {
		return required;
	}

	public int size() {
		return namedOpts.size();
	}
}

class NamedOptionWrapper {
	private boolean isRequired;
	private String helpString;
	private INamedOptionDefinition namedOpt;

	public NamedOptionWrapper( INamedOptionDefinition namedOpt, String helpString, boolean isRequired) {
		this.setNamedOpt(namedOpt);
		this.setRequired(isRequired);
		this.setHelpString(helpString);
	}

	public String getHelpString() {
		return helpString;
	}

	public void setHelpString(String helpString) {
		this.helpString = helpString;
	}

	public INamedOptionDefinition getNamedOpt() {
		return namedOpt;
	}

	public void setNamedOpt(INamedOptionDefinition namedOpt) {
		this.namedOpt = namedOpt;
	}

	public boolean isRequired() {
		return isRequired;
	}

	public void setRequired(boolean isRequired) {
		this.isRequired = isRequired;
	}
}

class PositionalDefinitionWrapper {
	private String helpString;
	private IPositionalOptionDefinition positionalOpt;

	public PositionalDefinitionWrapper( IPositionalOptionDefinition namedOpt, String helpString) {
		this.setPositionalOpt(namedOpt);
		this.setHelpString(helpString);
	}

	public String getHelpString() {
		return helpString;
	}

	private void setHelpString(String helpString) {
		this.helpString = helpString;
	}

	public IPositionalOptionDefinition getPositionalOpt() {
		return positionalOpt;
	}

	private void setPositionalOpt(IPositionalOptionDefinition positionalOpt) {
		this.positionalOpt = positionalOpt;
	}

	
}