/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.rtc.cli.infrastructure.internal.parser.exceptions;

import org.eclipse.osgi.util.NLS;

import com.ibm.team.rtc.cli.infrastructure.internal.Messages;
import com.ibm.team.rtc.cli.infrastructure.internal.parser.IOptionDefinition;

/**
 * Indicates that a caller attempted to add an option that conflicts with an
 * option that has already been installed.
 */
public class ConflictingOptionException extends Exception {
    private static final long serialVersionUID = 1L;

    final IOptionDefinition candidate;
    final IOptionDefinition alreadyInstalled;
    final String field;
    final String value;

    /**
     * @param opt
     *            The new option that cannot be installed.
     * @param other
     *            The installed option that has a conflicting field.
     * 
     * @param field
     *            The field that is conflicting.
     */
    public ConflictingOptionException(IOptionDefinition candidate,
            IOptionDefinition alreadyInstalled, String field, String value) {
        super();

        this.candidate = candidate;
        this.alreadyInstalled = alreadyInstalled;

        this.field = field;
        this.value = value;
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.lang.Throwable#getMessage()
     */
    @Override
    public String getMessage() {
        return NLS.bind(Messages.ConflictingOptionException_0, new Object[] { field, candidate,
                value });
    }

    public String getField() {
        return field;
    }

    public String getValue() {
        return value;
    }

    public IOptionDefinition getCandidate() {
        return candidate;
    }

    public IOptionDefinition getAlreadyInstalled() {
        return alreadyInstalled;
    }
}
