/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.rtc.cli.infrastructure.internal.util;

import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintStream;
import java.nio.charset.Charset;

/**
 * Behaves similar to a {@link PrintStream}, but allows an arbitrary amount of
 * indent on each line.
 */
public class IndentingPrintStream {

    public static final int INDENT = 2;

	final OutputStream stream;

    final String newline = System.getProperty("line.separator"); //$NON-NLS-1$

    Exception exception = null;

    final int indent;

    public IndentingPrintStream(OutputStream s) {
        stream = s;
        indent = 0;
    }

    public IndentingPrintStream(OutputStream s, int indent) {
        stream = s;
        this.indent = indent;
    }

    public int getIndent() {
        return indent;
    }

    /** Print a line with the current level of indent. */
    public void println(CharSequence s) {
        if (s == null)
            return;

        try {
            stream.write(createString(s).append(newline).toString()
                    .getBytes(Charset.defaultCharset().name()));
            stream.flush();
        } catch (IOException e) {
            exception = e;
        }
    }

    public void printIndent() {
        StringBuffer buf = new StringBuffer();

        for (int i = 0; i < indent; i++) {
            buf.append(" "); //$NON-NLS-1$
        }

        print(buf);
    }

    public void print(CharSequence s) {
        if (s == null)
            return;

        try {
            stream.write(s.toString().getBytes(Charset.defaultCharset().name()));
        } catch (IOException e) {
            exception = e;
        }
    }

    public void println() {
        try {
            stream.write(newline.getBytes(Charset.defaultCharset().name()));
            stream.flush();
        } catch (IOException e) {
            exception = e;
        }
    }

    public IndentingPrintStream indent() {
        return new IndentingPrintStream(stream, indent + INDENT);
    }

    protected StringBuffer createString(CharSequence toWrite) {
        StringBuffer buf = new StringBuffer(indent + toWrite.length());
        for (int i = 0; i < indent; i++) {
            buf.append(" "); //$NON-NLS-1$
        }

        buf.append(toWrite);

        return buf;
    }
}
