/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.rtc.cli.infrastructure.internal.util;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

public class Pair<T1, T2> {
    private T1 first;
    private T2 second;
    public Pair(T1 first, T2 second) {
        super();
        this.first = first;
        this.second = second;
    }
    
    public static <T1, T2> Pair<T1, T2> create(T1 first, T2 second) {
        return new Pair<T1, T2>(first, second);
    }
    
    public T1 getFirst() {
        return first;
    }

    public T2 getSecond() {
        return second;
    }

    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + ((first == null) ? 0 : first.hashCode());
        result = prime * result + ((second == null) ? 0 : second.hashCode());
        return result;
    }
    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null)
            return false;
        if (getClass() != obj.getClass())
            return false;
        final Pair other = (Pair) obj;
        if (first == null) {
            if (other.first != null)
                return false;
        } else if (!first.equals(other.first))
            return false;
        if (second == null) {
            if (other.second != null)
                return false;
        } else if (!second.equals(other.second))
            return false;
        return true;
    }
    
    public static <T1, T2> List<T1> getFirst(Collection<Pair<T1, T2>> toGet) {
    	ArrayList<T1> result = new ArrayList<T1>();
    	
    	for (Pair<T1, T2> next : toGet) {
    		result.add(next.getFirst());
    	}
    	
    	return result;
    }
    
    public static <T1, T2> List<T2> getSecond(Collection<Pair<T1, T2>> toGet) {
        ArrayList<T2> result = new ArrayList<T2>();
        
        for (Pair<T1, T2> next : toGet) {
            result.add(next.getSecond());
        }
        
        return result;
    }
}
