/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.rtc.cli.infrastructure.internal.util;

import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.PushbackReader;
import java.nio.charset.Charset;

/**
 * Read a password on stdin, "hide" the echo'd input.  
 */
public class PasswordReader {

	public static void main(String[] args) throws IOException {
		String pw = run("password:", System.in, System.out); //$NON-NLS-1$
		
		System.out.println("got: " + pw); //$NON-NLS-1$
	}
	
	private static class Overwriter implements Runnable {
		Thread thread;
		private OutputStream out;
		volatile boolean running = true;
		
		public Overwriter(OutputStream out) {
			this.out = out;
		}
		
		public void start() {
			thread = new Thread(this);
			thread.setPriority(Thread.MAX_PRIORITY);
			thread.setName("Password masker"); //$NON-NLS-1$
			thread.start();
		}

		public void stop() throws InterruptedException {
			running = false;
			thread.join();
		}
		
		
		public void run() {
			try {
				out.write(" ".getBytes(Charset.defaultCharset().name())); //$NON-NLS-1$
			} catch (IOException e1) {
				return;
			}
			
			while (running) {
				try {
					out.write("\b ".getBytes(Charset.defaultCharset().name())); //$NON-NLS-1$
					out.flush();
				} catch (IOException e) {
					break;
				} 

				try {
					Thread.sleep(1);
				} catch (InterruptedException e) {
				}
			}
		}
	}
	
	public static String run(String prompt, InputStream input, OutputStream output) throws IOException {
		StringBuffer text = new StringBuffer();
		
		PushbackReader in = new PushbackReader(new InputStreamReader(input, Charset.defaultCharset().name()));
		
		output.write(prompt.getBytes(Charset.defaultCharset().name()));
		output.flush();
		
		Overwriter overwriter = new Overwriter(output);
		overwriter.start();
		
		try {
			while (true) {
				int c = in.read();

				// Handle end of stream and *nix newline
				if (c == -1 || c == '\n') {
					break;
				}
				
				// Handle windows newline
				if (c == '\r') {
					int next = in.read();
					if (next == '\n' || next == -1) {
						break;
					}
					else {
						in.unread(next);
					}
				}
				
				text.append((char)c);
			}
		} catch (IOException e) {
		  // see bug 119109
		} finally {
			if (overwriter != null) {
				try {
					overwriter.stop();
				} catch (InterruptedException e) {
				    // Don't care
				}
			}
		}
		
		return text.toString();
	}
}
